@@ -1,5 +1,4 @@
 #![cfg_attr(not(test), no_std)]
-#![cfg_attr(all(feature = "nightly"), feature(allow_internal_unstable, macro_vis_matcher))]
 #![forbid(missing_docs)]
 
 //! A library for defining enums that can be used in compact bit sets. It supports enums up to 128
@@ -7,48 +6,33 @@
 //!
 //! # Defining enums for use with EnumSet
 //!
-//! Enums to be used with [`EnumSet`] should be defined through the [`enum_set_type!`] macro:
+//! Enums to be used with [`EnumSet`] should be defined using `#[derive(EnumSetType)]`:
 //!
 //! ```rust
-//! # #[macro_use] extern crate enumset;
 //! # use enumset::*;
-//! enum_set_type! {
-//!     /// Documentation for the enum
-//!     pub enum Enum {
-//!         A, B, C, D, E, F,
-//!         #[doc(hidden)] G,
-//!     }
+//! #[derive(EnumSetType, Copy, Clone, Debug)]
+//! pub enum Enum {
+//!    A, B, C, D, E, F, G,
 //! }
-//! # fn main() { }
 //! ```
 //!
 //! # Working with EnumSets
 //!
 //! EnumSets can be constructed via [`EnumSet::new()`] like a normal set. In addition,
-//! [`enum_set_type!`] creates operator overloads that allow you to create EnumSets like so:
+//! `#[derive(EnumSetType)]` creates operator overloads that allow you to create EnumSets like so:
 //!
 //! ```rust
-//! # #[macro_use] extern crate enumset;
 //! # use enumset::*;
-//! # enum_set_type! {
-//! #      pub enum Enum {
-//! #        A, B, C, D, E, F, G,
-//! #    }
-//! # }
+//! # #[derive(EnumSetType, Copy, Clone, Debug)] pub enum Enum { A, B, C, D, E, F, G }
 //! let new_set = Enum::A | Enum::C | Enum::G;
 //! assert_eq!(new_set.len(), 3);
 //! ```
 //!
 //! All bitwise operations you would expect to work on bitsets also work on both EnumSets and
-//! enums wrapped with [`enum_set_type!`]:
+//! enums with `#[derive(EnumSetType)]`:
 //! ```
-//! # #[macro_use] extern crate enumset;
 //! # use enumset::*;
-//! # enum_set_type! {
-//! #      pub enum Enum {
-//! #        A, B, C, D, E, F, G,
-//! #    }
-//! # }
+//! # #[derive(EnumSetType, Copy, Clone, Debug)] pub enum Enum { A, B, C, D, E, F, G }
 //! // Intersection of sets
 //! assert_eq!((Enum::A | Enum::B) & Enum::C, EnumSet::empty());
 //! assert_eq!((Enum::A | Enum::B) & Enum::A, Enum::A);
@@ -68,11 +52,8 @@
 //! The [`enum_set!`] macro allows you to create EnumSets in constant contexts:
 //!
 //! ```rust
-//! # #[macro_use] extern crate enumset;
 //! # use enumset::*;
-//! # enum_set_type! {
-//! #     enum Enum { A, B, C }
-//! # }
+//! # #[derive(EnumSetType, Copy, Clone, Debug)] pub enum Enum { A, B, C, D, E, F, G }
 //! const CONST_SET: EnumSet<Enum> = enum_set!(Enum::A | Enum::B);
 //! assert_eq!(CONST_SET, Enum::A | Enum::B);
 //! ```
@@ -80,13 +61,8 @@
 //! Mutable operations on the [`EnumSet`] otherwise work basically as expected:
 //!
 //! ```rust
-//! # #[macro_use] extern crate enumset;
 //! # use enumset::*;
-//! # enum_set_type! {
-//! #      pub enum Enum {
-//! #        A, B, C, D, E, F, G,
-//! #    }
-//! # }
+//! # #[derive(EnumSetType, Copy, Clone, Debug)] pub enum Enum { A, B, C, D, E, F, G }
 //! let mut set = EnumSet::new();
 //! set.insert(Enum::A);
 //! set.insert_all(Enum::E | Enum::G);
@@ -95,45 +71,145 @@
 //! assert_eq!(set, Enum::A | Enum::E | Enum::G);
 //! ```
 
-#[cfg(test)]
-extern crate core;
+#[cfg(test)] extern crate core;
+extern crate enumset_derive;
 extern crate num_traits;
 
+pub use enumset_derive::*;
+mod enumset { pub use super::*; }
+
+use core::cmp::Ordering;
 use core::fmt;
 use core::fmt::{Debug, Formatter};
-use core::hash::Hash;
+use core::hash::{Hash, Hasher};
 use core::ops::*;
 
 use num_traits::*;
 
 #[doc(hidden)]
-/// The trait used to define enum types.
-/// This is **NOT** public API and may change at any time.
-pub unsafe trait EnumSetType : Copy + Ord + Eq + Hash {
-    type Repr: PrimInt + ToPrimitive + FromPrimitive + WrappingSub + CheckedShl + Debug + Hash;
-    const VARIANT_COUNT: u8;
+/// Everything in this module is internal API and may change at any time.
+pub mod internal {
+    use super::*;
 
-    fn enum_into_u8(self) -> u8;
-    unsafe fn enum_from_u8(val: u8) -> Self;
+    #[doc(hidden)]
+    /// A struct used to type check [`enum_set!`].
+    pub struct EnumSetSameTypeHack<'a, T: EnumSetType + 'static> {
+        pub unified: &'a [T],
+        pub enum_set: EnumSet<T>,
+    }
+
+    /// A reexport of core to allow our macros to be generic to std vs core.
+    pub extern crate core;
 }
 
-#[doc(hidden)]
-/// A struct used to type check [`enum_set!`].
-/// This is **NOT** public API and may change at any time.
-pub struct EnumSetSameTypeHack<'a, T: EnumSetType + 'static> {
-    pub unified: &'a [T],
-    pub enum_set: EnumSet<T>,
+mod private {
+    use super::*;
+    pub trait EnumSetTypeRepr : PrimInt + FromPrimitive + WrappingSub + CheckedShl + Debug + Hash {
+        const WIDTH: u8;
+    }
+    macro_rules! prim {
+        ($name:ty, $width:expr) => {
+            impl EnumSetTypeRepr for $name {
+                const WIDTH: u8 = $width;
+            }
+        }
+    }
+    prim!(u8  , 8  );
+    prim!(u16 , 16 );
+    prim!(u32 , 32 );
+    prim!(u64 , 64 );
+    prim!(u128, 128);
+}
+use private::EnumSetTypeRepr;
+
+/// The trait used to define enum types that may be used with [`EnumSet`].
+///
+/// This trait should be implemented using `#[derive(EnumSetType)]`. Its internal structure is
+/// not currently stable, and may change at any time.
+///
+/// # Custom Derive
+///
+/// The custom derive for `EnumSetType` automatically creates implementations of [`PartialEq`],
+/// [`Sub`], [`BitAnd`], [`BitOr`], [`BitXor`], and [`Not`] allowing the enum to be used as
+/// if it were an [`EnumSet`] in expressions. This can be disabled by adding an `#[enumset_no_ops]`
+/// annotation to the enum.
+///
+/// Any C-like enum is supported, as long as there are no more than 128 variants in the enum,
+/// and no variant discriminator is larger than 127.
+///
+/// # Examples
+///
+/// Deriving a plain EnumSetType:
+///
+/// ```rust
+/// # use enumset::*;
+/// #[derive(EnumSetType, Copy, Clone, Debug)]
+/// pub enum Enum {
+///    A, B, C, D, E, F, G,
+/// }
+/// ```
+///
+/// Deriving a sparse EnumSetType:
+///
+/// ```rust
+/// # use enumset::*;
+/// #[derive(EnumSetType, Copy, Clone, Debug)]
+/// pub enum SparseEnum {
+///    A = 10, B = 20, C = 30, D = 127,
+/// }
+/// ```
+///
+/// Deriving an EnumSetType without adding ops:
+///
+/// ```rust
+/// # use enumset::*;
+/// #[derive(EnumSetType, Copy, Clone, Debug)]
+/// #[enumset_no_ops]
+/// pub enum NoOpsEnum {
+///    A, B, C, D, E, F, G,
+/// }
+/// ```
+pub unsafe trait EnumSetType: Copy {
+    #[doc(hidden)] type Repr: EnumSetTypeRepr;
+    #[doc(hidden)] const ALL_BITS: Self::Repr;
+    #[doc(hidden)] fn enum_into_u8(self) -> u8;
+    #[doc(hidden)] unsafe fn enum_from_u8(val: u8) -> Self;
 }
 
-/// An efficient set type for enums created with the [`enum_set_type!`](./macro.enum_set_type.html)
-/// macro.
-#[derive(Copy, Clone, PartialOrd, Ord, PartialEq, Eq, Hash)]
-pub struct EnumSet<T : EnumSetType> { 
+/// An efficient set type for enums.
+pub struct EnumSet<T : EnumSetType> {
     #[doc(hidden)]
     /// This is public due to the [`enum_set!`] macro.
     /// This is **NOT** public API and may change at any time.
     pub __enumset_underlying: T::Repr
 }
+impl <T : EnumSetType> PartialOrd for EnumSet<T> {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        self.__enumset_underlying.partial_cmp(&other.__enumset_underlying)
+    }
+}
+impl <T : EnumSetType> Ord for EnumSet<T> {
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.__enumset_underlying.cmp(&other.__enumset_underlying)
+    }
+}
+impl <T : EnumSetType> PartialEq for EnumSet<T> {
+    fn eq(&self, other: &Self) -> bool {
+        self.__enumset_underlying.eq(&other.__enumset_underlying)
+    }
+}
+impl <T : EnumSetType> Eq for EnumSet<T> { }
+impl <T : EnumSetType> Hash for EnumSet<T> {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.__enumset_underlying.hash(state)
+    }
+}
+impl <T : EnumSetType> Clone for EnumSet<T> {
+    fn clone(&self) -> Self {
+        *self
+    }
+}
+impl <T : EnumSetType> Copy for EnumSet<T> { }
 impl <T : EnumSetType> EnumSet<T> {
     fn mask(bit: u8) -> T::Repr {
         Shl::<usize>::shl(T::Repr::one(), bit as usize)
@@ -143,14 +219,14 @@ impl <T : EnumSetType> EnumSet<T> {
         self.__enumset_underlying & mask == mask
     }
     fn partial_bits(bits: u8) -> T::Repr {
-        assert!(bits != 0 && bits <= T::VARIANT_COUNT);
         T::Repr::one().checked_shl(bits.into())
             .unwrap_or(T::Repr::zero())
             .wrapping_sub(&T::Repr::one())
     }
+
     // Returns all bits valid for the enum
     fn all_bits() -> T::Repr {
-        Self::partial_bits(T::VARIANT_COUNT)
+        T::ALL_BITS
     }
 
     /// Returns an empty set.
@@ -174,8 +250,19 @@ impl <T : EnumSetType> EnumSet<T> {
 
     /// Total number of bits this enumset uses. Note that the actual amount of space used is
     /// rounded up to the next highest integer type (`u8`, `u16`, `u32`, `u64`, or `u128`).
+    ///
+    /// This is the same as [`EnumSet::variant_count`] except in enums with "sparse" variants.
+    /// (e.g. `enum Foo { A = 10, B = 20 }`)
     pub fn bit_width() -> u8 {
-        T::VARIANT_COUNT as u8
+        T::Repr::WIDTH - T::ALL_BITS.leading_zeros() as u8
+    }
+
+    /// The number of valid variants in this enumset.
+    ///
+    /// This is the same as [`EnumSet::bit_width`] except in enums with "sparse" variants.
+    /// (e.g. `enum Foo { A = 10, B = 20 }`)
+    pub fn variant_count() -> u8 {
+        T::ALL_BITS.count_ones() as u8
     }
 
     /// Returns the raw bits of this set
@@ -369,7 +456,7 @@ impl <T : EnumSetType> Iterator for EnumSetIter<T> {
     type Item = T;
 
     fn next(&mut self) -> Option<Self::Item> {
-        while self.1 < T::VARIANT_COUNT {
+        while self.1 < EnumSet::<T>::bit_width() {
             let bit = self.1;
             self.1 += 1;
             if self.0.has_bit(bit) {
@@ -385,227 +472,25 @@ impl <T : EnumSetType> Iterator for EnumSetIter<T> {
     }
 }
 
-#[macro_export]
-#[doc(hidden)]
-macro_rules! enum_set_type_internal_count_variants {
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $_10:ident $_11:ident $_12:ident $_13:ident $_14:ident $_15:ident $_16:ident $_17:ident
-        $_20:ident $_21:ident $_22:ident $_23:ident $_24:ident $_25:ident $_26:ident $_27:ident
-        $_30:ident $_31:ident $_32:ident $_33:ident $_34:ident $_35:ident $_36:ident $_37:ident
-        $_40:ident $_41:ident $_42:ident $_43:ident $_44:ident $_45:ident $_46:ident $_47:ident
-        $_50:ident $_51:ident $_52:ident $_53:ident $_54:ident $_55:ident $_56:ident $_57:ident
-        $_60:ident $_61:ident $_62:ident $_63:ident $_64:ident $_65:ident $_66:ident $_67:ident
-        $_70:ident $_71:ident $_72:ident $_73:ident $_74:ident $_75:ident $_76:ident $_77:ident
-        $_80:ident $_81:ident $_82:ident $_83:ident $_84:ident $_85:ident $_86:ident $_87:ident
-        $_90:ident $_91:ident $_92:ident $_93:ident $_94:ident $_95:ident $_96:ident $_97:ident
-        $_a0:ident $_a1:ident $_a2:ident $_a3:ident $_a4:ident $_a5:ident $_a6:ident $_a7:ident
-        $_b0:ident $_b1:ident $_b2:ident $_b3:ident $_b4:ident $_b5:ident $_b6:ident $_b7:ident
-        $_c0:ident $_c1:ident $_c2:ident $_c3:ident $_c4:ident $_c5:ident $_c6:ident $_c7:ident
-        $_d0:ident $_d1:ident $_d2:ident $_d3:ident $_d4:ident $_d5:ident $_d6:ident $_d7:ident
-        $_e0:ident $_e1:ident $_e2:ident $_e3:ident $_e4:ident $_e5:ident $_e6:ident $_e7:ident
-        $_f0:ident $_f1:ident $_f2:ident $_f3:ident $_f4:ident $_f5:ident $_f6:ident $_f7:ident
-        $($rest:ident)+
-    ) => {
-        compile_error!("enum_set_type! can only accept up to 128 variants.")
-    };
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $_10:ident $_11:ident $_12:ident $_13:ident $_14:ident $_15:ident $_16:ident $_17:ident
-        $_20:ident $_21:ident $_22:ident $_23:ident $_24:ident $_25:ident $_26:ident $_27:ident
-        $_30:ident $_31:ident $_32:ident $_33:ident $_34:ident $_35:ident $_36:ident $_37:ident
-        $_40:ident $_41:ident $_42:ident $_43:ident $_44:ident $_45:ident $_46:ident $_47:ident
-        $_50:ident $_51:ident $_52:ident $_53:ident $_54:ident $_55:ident $_56:ident $_57:ident
-        $_60:ident $_61:ident $_62:ident $_63:ident $_64:ident $_65:ident $_66:ident $_67:ident
-        $_70:ident $_71:ident $_72:ident $_73:ident $_74:ident $_75:ident $_76:ident $_77:ident
-        $($rest:ident)+
-    ) => {
-        enum_set_type_internal! { @$next u128 $($args)* }
-    };
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $_10:ident $_11:ident $_12:ident $_13:ident $_14:ident $_15:ident $_16:ident $_17:ident
-        $_20:ident $_21:ident $_22:ident $_23:ident $_24:ident $_25:ident $_26:ident $_27:ident
-        $_30:ident $_31:ident $_32:ident $_33:ident $_34:ident $_35:ident $_36:ident $_37:ident
-        $($rest:ident)+
-    ) => {
-        enum_set_type_internal! { @$next u64 $($args)* }
-    };
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $_10:ident $_11:ident $_12:ident $_13:ident $_14:ident $_15:ident $_16:ident $_17:ident
-        $($rest:ident)+
-    ) => {
-        enum_set_type_internal! { @$next u32 $($args)* }
-    };
-    ($next:ident ($($args:tt)*)
-        $_00:ident $_01:ident $_02:ident $_03:ident $_04:ident $_05:ident $_06:ident $_07:ident
-        $($rest:ident)+
-    ) => {
-        enum_set_type_internal! { @$next u16 $($args)* }
-    };
-    ($next:ident ($($args:tt)*) $($rest:ident)*) => {
-        enum_set_type_internal! { @$next u8 $($args)* }
-    };
-}
-
-#[macro_export]
-#[doc(hidden)]
-macro_rules! enum_set_type_internal {
-    // Counting functions
-    (@ident ($($random:tt)*) $value:expr) => { $value };
-    (@count $($value:tt)*) => {
-        0u8 $(+ enum_set_type_internal!(@ident ($value) 1u8))*
-    };
-
-    // Codegen
-    (@body $repr:ident ($(#[$enum_attr:meta])*) ($($vis:tt)*) $enum_name:ident {
-        $($(#[$attr:meta])* $variant:ident,)*
-    }) => {
-        $(#[$enum_attr])* #[repr(u8)]
-        #[derive(Copy, Clone, PartialOrd, Ord, PartialEq, Eq, Hash, Debug)]
-        $($vis)* enum $enum_name {
-            $($(#[$attr])* $variant,)*
-        }
-        unsafe impl $crate::EnumSetType for $enum_name {
-            type Repr = $repr;
-            const VARIANT_COUNT: u8 = enum_set_type_internal!(@count $($variant)*);
-
-            fn enum_into_u8(self) -> u8 {
-                self as u8
-            }
-            unsafe fn enum_from_u8(val: u8) -> Self {
-                ::std::mem::transmute(val)
-            }
-        }
-        impl <O : Into<$crate::EnumSet<$enum_name>>> ::std::ops::Sub<O> for $enum_name {
-            type Output = $crate::EnumSet<$enum_name>;
-            fn sub(self, other: O) -> Self::Output {
-                $crate::EnumSet::only(self) - other.into()
-            }
-        }
-        impl <O : Into<$crate::EnumSet<$enum_name>>> ::std::ops::BitAnd<O> for $enum_name {
-            type Output = $crate::EnumSet<$enum_name>;
-            fn bitand(self, other: O) -> Self::Output {
-                $crate::EnumSet::only(self) & other.into()
-            }
-        }
-        impl <O : Into<$crate::EnumSet<$enum_name>>> ::std::ops::BitOr<O> for $enum_name {
-            type Output = $crate::EnumSet<$enum_name>;
-            fn bitor(self, other: O) -> Self::Output {
-                $crate::EnumSet::only(self) | other.into()
-            }
-        }
-        impl <O : Into<$crate::EnumSet<$enum_name>>> ::std::ops::BitXor<O> for $enum_name {
-            type Output = $crate::EnumSet<$enum_name>;
-            fn bitxor(self, other: O) -> Self::Output {
-                $crate::EnumSet::only(self) ^ other.into()
-            }
-        }
-        impl ::std::ops::Not for $enum_name {
-            type Output = $crate::EnumSet<$enum_name>;
-            fn not(self) -> Self::Output {
-                !$crate::EnumSet::only(self)
-            }
-        }
-        impl ::std::cmp::PartialEq<$crate::EnumSet<$enum_name>> for $enum_name {
-            fn eq(&self, other: &$crate::EnumSet<$enum_name>) -> bool {
-                $crate::EnumSet::only(*self) == *other
-            }
-        }
-    };
-}
-
 /// Defines enums which can be used with EnumSet.
 ///
-/// While attributes and documentation can be attached to the enum variants, the variants may not
-/// contain data.
-///
 /// [`Copy`], [`Clone`], [`PartialOrd`], [`Ord`], [`PartialEq`], [`Eq`], [`Hash`], [`Debug`],
 /// [`Sub`], [`BitAnd`], [`BitOr`], [`BitXor`], and [`Not`] are automatically derived for the enum.
 ///
 /// These impls, in general, behave as if the enum variant was an [`EnumSet`] with a single value,
 /// as those created by [`EnumSet::only`].
-///
-/// # Examples
-///
-/// ```rust
-/// # #[macro_use] extern crate enumset;
-/// # use enumset::*;
-/// enum_set_type! {
-///     enum Enum {
-///         A, B, C, D, E, F, G
-///     }
-///
-///     /// Documentation
-///     pub enum Enum2 {
-///         A, B, C, D, E, F, G,
-///         #[doc(hidden)] __NonExhaustive,
-///     }
-/// }
-/// # fn main() { }
-/// ```
 #[macro_export]
-#[cfg(not(feature = "nightly"))]
-macro_rules! enum_set_type {
-    ($(#[$enum_attr:meta])* pub enum $enum_name:ident {
-        $($(#[$attr:meta])* $variant:ident),* $(,)*
-    } $($rest:tt)*) => {
-        enum_set_type_internal_count_variants!(body (($(#[$enum_attr])*) (pub) $enum_name {
-            $($(#[$attr])* $variant,)*
-        }) $($variant)*);
-        enum_set_type!($($rest)*);
-    };
-    ($(#[$enum_attr:meta])* enum $enum_name:ident {
-        $($(#[$attr:meta])* $variant:ident),* $(,)*
-    } $($rest:tt)*) => {
-        enum_set_type_internal_count_variants!(body (($(#[$enum_attr])*) () $enum_name {
-            $($(#[$attr])* $variant,)*
-        }) $($variant)*);
-        enum_set_type!($($rest)*);
-    };
-    () => { };
-}
-
-/// Defines enums which can be used with EnumSet.
-///
-/// While attributes and documentation can be attached to the enum variants, the variants may not
-/// contain data.
-///
-/// [`Copy`], [`Clone`], [`PartialOrd`], [`Ord`], [`PartialEq`], [`Eq`], [`Hash`], [`Debug`],
-/// [`Sub`], [`BitAnd`], [`BitOr`], [`BitXor`], and [`Not`] are automatically derived for the enum.
-///
-/// These impls, in general, behave as if the enum variant was an [`EnumSet`] with a single value,
-/// as those created by [`EnumSet::only`].
-///
-/// # Examples
-///
-/// ```rust
-/// # #[macro_use] extern crate enumset;
-/// # use enumset::*;
-/// enum_set_type! {
-///     enum Enum {
-///         A, B, C, D, E, F, G
-///     }
-///
-///     /// Documentation
-///     pub enum Enum2 {
-///         A, B, C, D, E, F, G,
-///         #[doc(hidden)] __NonExhaustive,
-///     }
-/// }
-/// # fn main() { }
-/// ```
-#[macro_export]
-#[cfg(feature = "nightly")]
-#[allow_internal_unstable]
+#[deprecated(since = "0.3.13", note = "Use `#[derive(EnumSetType)] instead.")]
 macro_rules! enum_set_type {
     ($(#[$enum_attr:meta])* $vis:vis enum $enum_name:ident {
         $($(#[$attr:meta])* $variant:ident),* $(,)*
     } $($rest:tt)*) => {
-        enum_set_type_internal_count_variants!(body (($(#[$enum_attr])*) ($vis) $enum_name {
+        $(#[$enum_attr])* #[repr(u8)]
+        #[derive($crate::EnumSetType, Copy, Clone, PartialOrd, Ord, PartialEq, Eq, Hash, Debug)]
+        $vis enum $enum_name {
             $($(#[$attr])* $variant,)*
-        }) $($variant)*);
+        }
+
         enum_set_type!($($rest)*);
     };
     () => { };
@@ -622,32 +507,22 @@ macro_rules! enum_set_type {
 /// # Examples
 ///
 /// ```rust
-/// # #[macro_use] extern crate enumset;
 /// # use enumset::*;
-/// # enum_set_type! {
-/// #     enum Enum { A, B, C }
-/// # }
-/// # fn main() {
+/// # #[derive(EnumSetType, Copy, Clone, Debug)] enum Enum { A, B, C }
 /// const CONST_SET: EnumSet<Enum> = enum_set!(Enum::A | Enum::B);
 /// assert_eq!(CONST_SET, Enum::A | Enum::B);
 ///
 /// const EXPLICIT_CONST_SET: EnumSet<Enum> = enum_set!(Enum, Enum::A | Enum::B);
 /// assert_eq!(EXPLICIT_CONST_SET, Enum::A | Enum::B);
-/// # }
 /// ```
 ///
 /// This macro is strongly typed. For example, the following will not compile:
 ///
 /// ```compile_fail
-/// # #[macro_use] extern crate enumset;
 /// # use enumset::*;
-/// # enum_set_type! {
-/// #     enum Enum { A, B, C }
-/// #     enum Enum2 { A, B, C }
-/// # }
-/// # fn main() {
+/// # #[derive(EnumSetType, Copy, Clone, Debug)] enum Enum { A, B, C }
+/// # #[derive(EnumSetType, Copy, Clone, Debug)] enum Enum2 { A, B, C }
 /// let type_error = enum_set!(Enum::A | Enum2::B);
-/// # }
 /// ```
 #[macro_export]
 macro_rules! enum_set {
@@ -655,7 +530,7 @@ macro_rules! enum_set {
         $crate::EnumSet { __enumset_underlying: 0 }
     };
     ($($value:path)|* $(|)*) => {
-        $crate::EnumSetSameTypeHack {
+        $crate::internal::EnumSetSameTypeHack {
             unified: &[$($value,)*],
             enum_set: $crate::EnumSet {
                 __enumset_underlying: 0 $(| (1 << ($value as u8)))*
@@ -675,49 +550,47 @@ mod test {
     use super::*;
 
     mod enums {
-        enum_set_type! {
-            pub enum SmallEnum {
-                A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
-            }
-            pub enum LargeEnum {
-                _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
-                _10,  _11,  _12,  _13,  _14,  _15,  _16,  _17,
-                _20,  _21,  _22,  _23,  _24,  _25,  _26,  _27,
-                _30,  _31,  _32,  _33,  _34,  _35,  _36,  _37,
-                _40,  _41,  _42,  _43,  _44,  _45,  _46,  _47,
-                _50,  _51,  _52,  _53,  _54,  _55,  _56,  _57,
-                _60,  _61,  _62,  _63,  _64,  _65,  _66,  _67,
-                _70,  _71,  _72,  _73,  _74,  _75,  _76,  _77,
-                A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
-            }
-            pub enum Enum8 {
-                A, B, C, D, E, F, G, H,
-            }
-            pub enum Enum128 {
-                A, B, C, D, E, F, G, H, _8, _9, _10, _11, _12, _13, _14, _15,
-                _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31,
-                _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47,
-                _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62, _63,
-                _64, _65, _66, _67, _68, _69, _70, _71, _72, _73, _74, _75, _76, _77, _78, _79,
-                _80, _81, _82, _83, _84, _85, _86, _87, _88, _89, _90, _91, _92, _93, _94, _95,
-                _96, _97, _98, _99, _100, _101, _102, _103, _104, _105, _106, _107, _108, _109,
-                _110, _111, _112, _113, _114, _115, _116, _117, _118, _119, _120, _121, _122, _123,
-                _124,  _125, _126, _127,
-            }
+        #[derive(::EnumSetType, Copy, Clone, Debug)]
+        pub enum SmallEnum {
+            A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
+        }
+        #[derive(::EnumSetType, Copy, Clone, Debug)]
+        pub enum LargeEnum {
+            _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
+            _10,  _11,  _12,  _13,  _14,  _15,  _16,  _17,
+            _20,  _21,  _22,  _23,  _24,  _25,  _26,  _27,
+            _30,  _31,  _32,  _33,  _34,  _35,  _36,  _37,
+            _40,  _41,  _42,  _43,  _44,  _45,  _46,  _47,
+            _50,  _51,  _52,  _53,  _54,  _55,  _56,  _57,
+            _60,  _61,  _62,  _63,  _64,  _65,  _66,  _67,
+            _70,  _71,  _72,  _73,  _74,  _75,  _76,  _77,
+            A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
+        }
+        #[derive(::EnumSetType, Copy, Clone, Debug)]
+        pub enum Enum8 {
+            A, B, C, D, E, F, G, H,
+        }
+        #[derive(::EnumSetType, Copy, Clone, Debug)]
+        pub enum Enum128 {
+            A, B, C, D, E, F, G, H, _8, _9, _10, _11, _12, _13, _14, _15,
+            _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31,
+            _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47,
+            _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62, _63,
+            _64, _65, _66, _67, _68, _69, _70, _71, _72, _73, _74, _75, _76, _77, _78, _79,
+            _80, _81, _82, _83, _84, _85, _86, _87, _88, _89, _90, _91, _92, _93, _94, _95,
+            _96, _97, _98, _99, _100, _101, _102, _103, _104, _105, _106, _107, _108, _109,
+            _110, _111, _112, _113, _114, _115, _116, _117, _118, _119, _120, _121, _122,
+            _123, _124,  _125, _126, _127,
+        }
+        #[derive(::EnumSetType, Copy, Clone, Debug)]
+        pub enum SparseEnum {
+            A = 10, B = 20, C = 30, D = 40, E = 50, F = 60, G = 70, H = 80,
         }
     }
     use self::enums::*;
 
     macro_rules! test_variants {
-        ($enum_name:ident $variant_range:ident $all_empty_test:ident $($variant:ident,)*) => {
-            #[test]
-            fn $variant_range() {
-                let count = enum_set_type_internal!(@count u8 $($variant)*);
-                $(
-                    assert!(($enum_name::$variant as u8) < count);
-                )*
-            }
-
+        ($enum_name:ident $all_empty_test:ident $($variant:ident,)*) => {
             #[test]
             fn $all_empty_test() {
                 let all = EnumSet::<$enum_name>::all();
@@ -730,12 +603,10 @@ mod test {
             }
         }
     }
-
-    test_variants! { SmallEnum enum_variant_range_test enum_all_empty
+    test_variants! { SmallEnum small_enum_all_empty
         A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
     }
-
-    test_variants! { LargeEnum large_enum_variant_range_test large_enum_all_empty
+    test_variants! { LargeEnum large_enum_all_empty
         _00,  _01,  _02,  _03,  _04,  _05,  _06,  _07,
         _10,  _11,  _12,  _13,  _14,  _15,  _16,  _17,
         _20,  _21,  _22,  _23,  _24,  _25,  _26,  _27,
@@ -746,6 +617,9 @@ mod test {
         _70,  _71,  _72,  _73,  _74,  _75,  _76,  _77,
         A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z,
     }
+    test_variants! { SparseEnum sparse_enum_all_empty
+        A, B, C, D, E, F, G,
+    }
 
     macro_rules! test_enum {
         ($e:ident, $m:ident) => {
@@ -789,7 +663,7 @@ mod test {
 
                 #[test]
                 fn all_len() {
-                    assert_eq!(EnumSet::<$e>::all().len(), EnumSet::<$e>::bit_width() as usize)
+                    assert_eq!(EnumSet::<$e>::all().len(), EnumSet::<$e>::variant_count() as usize)
                 }
 
                 #[test]
@@ -846,7 +720,7 @@ mod test {
                 #[test]
                 #[should_panic]
                 fn too_many_bits() {
-                    if EnumSet::<$e>::bit_width() == 128 {
+                    if EnumSet::<$e>::variant_count() == 128 {
                         panic!("(test skipped)")
                     }
                     EnumSet::<$e>::from_bits(!0);
@@ -859,4 +733,5 @@ mod test {
     test_enum!(LargeEnum, large_enum);
     test_enum!(Enum8, enum8);
     test_enum!(Enum128, enum128);
+    test_enum!(SparseEnum, sparse_enum);
 }
